@@ -17,10 +17,18 @@ class LSDeconvolution : public DeconvolutionAlgorithm
 		LSDeconvolution();
 		~LSDeconvolution();
 		
-    virtual void ExecuteMajorIteration(ImageSet& dataImage, ImageSet& modelImage, const ao::uvector<const double*>& psfImages, size_t width, size_t height, bool& reachedMajorThreshold) final override
+		LSDeconvolution(const LSDeconvolution& source);
+		
+    virtual double ExecuteMajorIteration(ImageSet& dataImage, ImageSet& modelImage, const ao::uvector<const double*>& psfImages, size_t width, size_t height, bool& reachedMajorThreshold) final override
 		{
       _allocator = &dataImage.Allocator();
 			ExecuteMajorIteration(dataImage[0], modelImage[0], psfImages[0], width, height, reachedMajorThreshold);
+			return 0.0;
+		}
+		
+		virtual std::unique_ptr<DeconvolutionAlgorithm> Clone() const final override
+		{
+			return std::unique_ptr<DeconvolutionAlgorithm>(new LSDeconvolution(*this));
 		}
 		
 		void ExecuteMajorIteration(double* dataImage, double* modelImage, const double* psfImage, size_t width, size_t height, bool& reachedMajorThreshold)
