@@ -26,7 +26,10 @@ void LBeamImageMaker::Make(PrimaryBeamImageSet&)
 #include "../msproviders/msprovider.h"
 #include "../multibanddata.h"
 
+#include "lofarbeamkeywords.h"
+
 #include <StationResponse/LofarMetaDataUtil.h>
+#include <StationResponse/ITRFConverter.h>
 
 #include <casacore/ms/MeasurementSets/MSField.h>
 
@@ -45,7 +48,7 @@ void LBeamImageMaker::Make(PrimaryBeamImageSet&)
 
 using namespace LOFAR::StationResponse;
 
-static void dirToITRF(const casacore::MDirection& dir, casacore::MDirection::Convert& convert, vector3r_t& itrf);
+static void dirToITRFVector(const casacore::MDirection& dir, ITRFConverter& convert, vector3r_t& itrf);
 
 void LBeamImageMaker::Make(PrimaryBeamImageSet& beamImages)
 {
@@ -63,7 +66,7 @@ void LBeamImageMaker::Make(PrimaryBeamImageSet& beamImages)
 	{
 		const ImagingTableEntry::MSInfo& msInfo = _tableEntry->msData[msProviderInfo.msIndex];
 		const MSSelection& selection = *msProviderInfo.selection;
-		casacore::MeasurementSet& ms = msProviderInfo.provider->MS();
+		casacore::MeasurementSet ms = msProviderInfo.provider->MS();
 		MultiBandData band(ms.spectralWindow(), ms.dataDescription());
 		double centralFrequency = 0.0;
 		for(size_t dataDescId=0; dataDescId!=band.DataDescCount(); ++dataDescId)
@@ -72,7 +75,7 @@ void LBeamImageMaker::Make(PrimaryBeamImageSet& beamImages)
 			centralFrequency += subBand.CentreFrequency();
 		}
 		centralFrequency /= msInfo.bands.size();
-		makeBeamForMS(beamImages, *msProviderInfo.provider, msInfo, selection, centralFrequency);
+		makeBeamForMS(beamImages, *msProviderInfo.provider, selection, centralFrequency);
 	}
 
 	for(size_t i=0; i!=8; ++i)
@@ -90,18 +93,18 @@ void LBeamImageMaker::Make(PrimaryBeamImageSet& beamImages)
 		_allocator->Allocate(_width*_height, scratch);
 		for(size_t p=0; p!=8; ++p)
 		{
-			resampler.RunSingle(&beamImages[p][0], scratch.data());
+			resampler.Resample(&beamImages[p][0], scratch.data());
 			memcpy(&beamImages[p][0], scratch.data(), sizeof(double)*_width*_height);
 		}
 	}
 }
 
-void LBeamImageMaker::makeBeamForMS(PrimaryBeamImageSet& beamImages, MSProvider& msProvider, const ImagingTableEntry::MSInfo& msInfo, const MSSelection& selection, double centralFrequency)
+void LBeamImageMaker::makeBeamForMS(PrimaryBeamImageSet& beamImages, MSProvider& msProvider, const MSSelection& selection, double centralFrequency)
 {
 	/**
 		* Read some meta data from the measurement set
 		*/
-	casacore::MeasurementSet& ms = msProvider.MS();
+	casacore::MeasurementSet ms = msProvider.MS();
 	
 	casacore::MSAntenna aTable = ms.antenna();
 	if(aTable.nrow() == 0) throw std::runtime_error("No antennae in set");
@@ -116,9 +119,8 @@ void LBeamImageMaker::makeBeamForMS(PrimaryBeamImageSet& beamImages, MSProvider&
 		throw std::runtime_error("Set has multiple fields");
 	_delayDir = delayDirColumn(0);
 	//Logger::Debug << "Using delay direction: " << RaDecCoord::RaDecToString(_delayDir.getAngle().getValue()[0], _delayDir.getAngle().getValue()[1]) << '\n';
-	
-	casacore::ROScalarMeasColumn<casacore::MDirection> referenceDirColumn(fieldTable, casacore::MSField::columnName(casacore::MSFieldEnums::REFERENCE_DIR));
-	_referenceDir = referenceDirColumn(0);
+
+	LOFARBeamKeywords::GetPreappliedBeamDirection(ms, msProvider.DataColumnName(), _useDifferentialBeam, _preappliedDir);
 	
 	if(fieldTable.tableDesc().isColumn("LOFAR_TILE_BEAM_DIR")) {
 		casacore::ROArrayMeasColumn<casacore::MDirection> tileBeamDirColumn(fieldTable, "LOFAR_TILE_BEAM_DIR");
@@ -206,7 +208,7 @@ void LBeamImageMaker::makeBeamForMS(PrimaryBeamImageSet& beamImages, MSProvider&
 				imgPtr[i] = &singleImages[i][0];
 			}
 		
-			makeBeamSnapshot(stations, stationWeights, baselineWeights, imgPtr, timeEpoch.getValue().get()*86400.0, centralFrequency, centralFrequency, frame);
+			makeBeamSnapshot(stations, stationWeights, imgPtr, timeEpoch.getValue().get()*86400.0, centralFrequency, centralFrequency, frame);
 		
 			_totalWeightSum += intervalWeight;
 			for(size_t i=0; i!=8; ++i)
@@ -221,33 +223,32 @@ void LBeamImageMaker::makeBeamForMS(PrimaryBeamImageSet& beamImages, MSProvider&
 	}
 }
 
-static void dirToITRF(const casacore::MDirection& dir, casacore::MDirection::Convert& convert, vector3r_t& itrf)
+static void dirToITRFVector(const casacore::MDirection& dir, ITRFConverter& convert, vector3r_t& itrf)
 {
-	casacore::MDirection itrfDir = convert(dir);
+	casacore::MDirection itrfDir = convert.toDirection(dir);
 	casacore::Vector<double> itrfVal = itrfDir.getValue().getValue();
 	itrf[0] = itrfVal[0];
 	itrf[1] = itrfVal[1];
 	itrf[2] = itrfVal[2];
 }
 
-void LBeamImageMaker::makeBeamSnapshot(const std::vector<Station::Ptr>& stations, const ao::uvector<double>& weights, const WeightMatrix& baselineWeights, double** imgPtr, double time, double frequency, double subbandFrequency, const casacore::MeasFrame& frame)
+void LBeamImageMaker::makeBeamSnapshot(const std::vector<Station::Ptr>& stations, const ao::uvector<double>& weights, double** imgPtr, double time, double frequency, double subbandFrequency, const casacore::MeasFrame& frame)
 {
 	static const casacore::Unit radUnit("rad");
 	const casacore::MDirection::Ref itrfRef(casacore::MDirection::ITRF, frame);
 	const casacore::MDirection::Ref j2000Ref(casacore::MDirection::J2000, frame);
-	casacore::MDirection::Convert
-		j2000ToITRFRef(j2000Ref, itrfRef);
+	ITRFConverter converter(time);
 		
 	vector3r_t station0, tile0, diffBeamCentre;
-	dirToITRF(_delayDir, j2000ToITRFRef, station0);
-	dirToITRF(_tileBeamDir, j2000ToITRFRef, tile0);
+	dirToITRFVector(_delayDir, converter, station0);
+	dirToITRFVector(_tileBeamDir, converter, tile0);
 	
 	Logger::Debug << "Time=" << time << '\n';
 	
 	std::vector<MC2x2> inverseCentralGain;
 	if(_useDifferentialBeam)
 	{
-		dirToITRF(_referenceDir, j2000ToITRFRef, diffBeamCentre);
+		dirToITRFVector(_preappliedDir, converter, diffBeamCentre);
 		inverseCentralGain.resize(stations.size());
 		for(size_t a=0; a!=stations.size(); ++a)
 		{
@@ -279,7 +280,7 @@ void LBeamImageMaker::makeBeamSnapshot(const std::vector<Station::Ptr>& stations
 							j2000Ref);
 			
 			vector3r_t itrfDirection;
-			dirToITRF(imageDir, j2000ToITRFRef, itrfDirection);
+			dirToITRFVector(imageDir, converter, itrfDirection);
 			
 			std::vector<MC2x2> stationGains(stations.size());
 			for(size_t a=0; a!=stations.size(); ++a)
@@ -333,7 +334,7 @@ void LBeamImageMaker::makeBeamSnapshot(const std::vector<Station::Ptr>& stations
 
 void LBeamImageMaker::calculateStationWeights(const ImageWeights& imageWeights, double& totalWeight, ao::uvector<double>& weights, WeightMatrix& baselineWeights, MSProvider& msProvider, const MSSelection& selection, double endTime)
 {
-	casacore::MeasurementSet& ms = msProvider.MS();
+	casacore::MeasurementSet ms = msProvider.MS();
 	casacore::MSAntenna antTable(ms.antenna());
 	totalWeight = 0.0;
 	weights.assign(antTable.nrow(), 0.0);
