@@ -17,6 +17,8 @@ void MSProvider::copyData(std::complex<float>* dest, size_t startChannel, size_t
 	size_t polIndex;
 	if(polOut == Polarization::Instrumental)
 	{
+		if(polsIn.size() != 4)
+			throw std::runtime_error("This mode requires the four polarizations to be present in the measurement set");
 		for(size_t ch=0; ch!=selectedChannelCount*polsIn.size(); ++ch)
 		{
 			if(isfinite(*inPtr))
@@ -237,7 +239,9 @@ void MSProvider::copyWeights(NumType* dest, size_t startChannel, size_t endChann
 		for(size_t ch=0; ch!=selectedChannelCount * polsIn.size(); ++ch)
 		{
 			if(!*flagPtr && isfinite(*inPtr))
-				dest[ch] = *weightPtr;
+				// The factor of 4 is to be consistent with StokesI
+				// It is for having conjugate visibilities and because IDG doesn't separately count XX and YY visibilities
+				dest[ch] = *weightPtr * 4.0f; 
 			else
 				dest[ch] = 0.0f;
 			inPtr++;
@@ -568,7 +572,7 @@ void MSProvider::getRowRange(casacore::MeasurementSet& ms, const MSSelection& se
 	}
 }
 
-void MSProvider::getRowRangeAndIDMap(casacore::MeasurementSet& ms, const MSSelection& selection, size_t& startRow, size_t& endRow, const std::set<size_t>& dataDescIds, vector<size_t>& idToMSRow)
+void MSProvider::getRowRangeAndIDMap(casacore::MeasurementSet& ms, const MSSelection& selection, size_t& startRow, size_t& endRow, const std::set<size_t>& dataDescIds, std::vector<size_t>& idToMSRow)
 {
 	startRow = 0;
 	endRow = ms.nrow();
@@ -698,7 +702,7 @@ casacore::ArrayColumn<float> MSProvider::initializeImagingWeightColumn(casacore:
 	}
 }
 
-vector<PolarizationEnum> MSProvider::GetMSPolarizations(casacore::MeasurementSet& ms)
+std::vector<PolarizationEnum> MSProvider::GetMSPolarizations(casacore::MeasurementSet& ms)
 {
 	std::vector<PolarizationEnum> pols;
 	casacore::MSPolarization polTable(ms.polarization());
