@@ -252,20 +252,28 @@ void PartitionedMS::ReadWeights(float* buffer)
 	_weightFile.read(reinterpret_cast<char*>(buffer), _partHeader.channelCount * _polarizationCountInFile * sizeof(float));
 	_weightPtrIsOk = false;
 }
-
-std::string PartitionedMS::getPartPrefix(const std::string& msPathStr, size_t partIndex, PolarizationEnum pol, size_t dataDescId, const std::string& tempDir)
+std::string PartitionedMS::getFilenamePrefix(const std::string& msPathStr, const std::string& tempDir)
 {
 	boost::filesystem::path
-		msPath(msPathStr),
 		prefixPath;
 	if(tempDir.empty())
-		prefixPath = msPath;
-	else
+		prefixPath = msPathStr;
+	else {
+		std::string msPathCopy(msPathStr);
+		while(!msPathCopy.empty() && *msPathCopy.rbegin() == '/')
+			msPathCopy.resize(msPathCopy.size()-1);
+		boost::filesystem::path msPath(msPathCopy);
 		prefixPath = boost::filesystem::path(tempDir) / msPath.filename();
-		
+	}
 	std::string prefix(prefixPath.string());
 	while(!prefix.empty() && *prefix.rbegin() == '/')
 		prefix.resize(prefix.size()-1);
+	return prefix;
+}
+
+std::string PartitionedMS::getPartPrefix(const std::string& msPathStr, size_t partIndex, PolarizationEnum pol, size_t dataDescId, const std::string& tempDir)
+{
+	std::string prefix = getFilenamePrefix(msPathStr, tempDir);
 	
 	std::ostringstream partPrefix;
 	partPrefix << prefix << "-part";
@@ -281,16 +289,8 @@ std::string PartitionedMS::getPartPrefix(const std::string& msPathStr, size_t pa
 
 string PartitionedMS::getMetaFilename(const string& msPathStr, const std::string& tempDir, size_t dataDescId)
 {
-	boost::filesystem::path
-		msPath(msPathStr),
-		prefixPath;
-	if(tempDir.empty())
-		prefixPath = msPath;
-	else
-		prefixPath = boost::filesystem::path(tempDir) / msPath.filename();
-	std::string prefix(prefixPath.string());
-	while(!prefix.empty() && *prefix.rbegin() == '/')
-		prefix.resize(prefix.size()-1);
+	std::string prefix = getFilenamePrefix(msPathStr, tempDir);
+	
 	std::ostringstream s;
 	s << prefix << "-spw" << dataDescId << "-parted-meta.tmp";
 	return s.str();
@@ -382,7 +382,8 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 	const casacore::IPosition shape(rowProvider->DataShape());
 	size_t channelCount = shape[1];
 	
-	Logger::Info << "Reordering " << msPath << " into " << channelParts << " x " << polsOut.size() << " parts.\n";
+	if(settings.parallelReordering == 1)
+		Logger::Info << "Reordering " << msPath << " into " << channelParts << " x " << polsOut.size() << " parts.\n";
 
 	// Write header of meta file, one meta file for each data desc id
 	// TODO rather than writing we can just skip and write later
@@ -411,13 +412,17 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 	casacore::Array<std::complex<float>> dataArray(shape), modelArray(shape);
 	casacore::Array<float> weightSpectrumArray(shape);
 	casacore::Array<bool> flagArray(shape);
-	ProgressBar progress1("Reordering");
+	
+	std::unique_ptr<ProgressBar> progress1;
+	if(settings.parallelReordering == 1)
+		progress1.reset(new ProgressBar("Reordering"));
 	
 	size_t selectedRowsTotal = 0;
 	ao::uvector<size_t> selectedRowCountPerSpwIndex(selectedDataDescIds.size(), 0);
 	while(!rowProvider->AtEnd())
 	{
-		progress1.SetProgress(rowProvider->CurrentProgress(), rowProvider->TotalProgress());
+		if(progress1)
+			progress1->SetProgress(rowProvider->CurrentProgress(), rowProvider->TotalProgress());
 		
 		MetaRecord meta;
 		memset(&meta, 0, sizeof(MetaRecord));
@@ -478,7 +483,7 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 		
 		rowProvider->NextRow();
 	}
-	progress1.SetProgress(rowProvider->TotalProgress(), rowProvider->TotalProgress());
+	progress1.reset();
 	Logger::Debug << "Total selected rows: " << selectedRowsTotal << '\n';
 	rowProvider->OutputStatistics();
 	
@@ -504,7 +509,7 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 	fileIndex = 0;
 	dataBuffer.assign(channelCount * polarizationsPerFile, 0.0);
 	std::unique_ptr<ProgressBar> progress2;
-	if(includeModel && !initialModelRequired)
+	if(includeModel && !initialModelRequired && settings.parallelReordering == 1)
 		progress2.reset(new ProgressBar("Initializing model visibilities"));
 	for(size_t part=0; part!=channelParts; ++part)
 	{
@@ -533,7 +538,8 @@ PartitionedMS::Handle PartitionedMS::Partition(const string& msPath, const std::
 				for(size_t i=0; i!=selectedRowCount; ++i)
 				{
 					modelFile.write(reinterpret_cast<char*>(dataBuffer.data()), header.channelCount * sizeof(std::complex<float>) * polarizationsPerFile);
-					progress2->SetProgress(part*selectedRowCount + i, channelParts*selectedRowCount);
+					if(progress2)
+						progress2->SetProgress(part*selectedRowCount + i, channelParts*selectedRowCount);
 				}
 			}
 		}
@@ -693,15 +699,8 @@ PartitionedMS::Handle::HandleData::~HandleData()
 	}
 }
 
-void PartitionedMS::openMS()
+void PartitionedMS::MakeIdToMSRowMapping(std::vector<size_t>& idToMSRow)
 {
-	if(_ms == 0)
-		_ms.reset(new casacore::MeasurementSet(_msPath.data()));
-}
-
-void PartitionedMS::MakeIdToMSRowMapping(vector<size_t>& idToMSRow)
-{
-	openMS();
 	const MSSelection& selection = _handle._data->_selection;
 	std::map<size_t, size_t> dataDescIds;
 	getDataDescIdMap(dataDescIds, _handle._data->_channels);
@@ -709,10 +708,11 @@ void PartitionedMS::MakeIdToMSRowMapping(vector<size_t>& idToMSRow)
 	for(std::map<size_t, size_t>::const_iterator i=dataDescIds.begin(); i!=dataDescIds.end(); ++i)
 		dataDescIdSet.insert(i->first);
 	size_t startRow, endRow;
-	getRowRangeAndIDMap(*_ms, selection, startRow, endRow, dataDescIdSet, idToMSRow);
+	casacore::MeasurementSet ms = MS();
+	getRowRangeAndIDMap(ms, selection, startRow, endRow, dataDescIdSet, idToMSRow);
 }
 
-void PartitionedMS::getDataDescIdMap(std::map<size_t, size_t>& dataDescIds, const vector<PartitionedMS::ChannelRange>& channels)
+void PartitionedMS::getDataDescIdMap(std::map<size_t, size_t>& dataDescIds, const std::vector<PartitionedMS::ChannelRange>& channels)
 {
 	size_t spwIndex = 0;
 	for(size_t i=0; i!=channels.size(); ++i)
