@@ -31,6 +31,8 @@ public:
 		/** Construct empty pointer */
 		Ptr()
 			: _data(nullptr), _allocator(nullptr) { }
+		Ptr(std::nullptr_t)
+			: _data(nullptr), _allocator(nullptr) { }
 		/** Construct initialized pointer
 			* @param data databuffer
 			* @param allocator corresponding allocator
@@ -83,6 +85,7 @@ public:
 		}
 		/** Retrieve image element with given index. */
 		double& operator[](size_t index) const { return _data[index]; }
+		
 	private:
 		Ptr(const Ptr&) = delete;
 		void operator=(const Ptr&) = delete;
@@ -138,6 +141,11 @@ public:
 		ptr.reset(Allocate(size), *this);
 	}
 	
+	Ptr AllocatePtr(size_t size)
+	{
+		return Ptr(Allocate(size), *this);
+	}
+	
 	double* Allocate(size_t size)
 	{
 		if(size == 0) size=2;
@@ -301,12 +309,15 @@ private:
 			msg << "posix_memalign() failed when allocating " << size*sizeof(double) * 2 << " bytes: ";
 			switch(errVal)
 			{
-				case EINVAL:
-					msg << "the alignment argument was not a power of two, or was not a multiple of sizeof(void *)";
-				case ENOMEM:
-					msg << "there was insufficient memory to fulfill the allocation request.";
-				default:
-					msg << "an unknown error value was returned";
+			case EINVAL:
+				msg << "the alignment argument was not a power of two, or was not a multiple of sizeof(void *)";
+				break;
+			case ENOMEM:
+				msg << "there was insufficient memory to fulfill the allocation request.";
+				break;
+			default:
+				msg << "an unknown error value was returned";
+				break;
 			}
 			throw std::runtime_error(msg.str());
 		}
