@@ -12,7 +12,7 @@
 class ImageWeightCache
 {
 public:
-	ImageWeightCache(const WeightMode& weightMode, size_t imageWidth, size_t imageHeight, double pixelScaleX, double pixelScaleY, double minUVInLambda, double maxUVInLambda, double rankFilterLevel, size_t rankFilterSize) :
+	ImageWeightCache(const WeightMode& weightMode, size_t imageWidth, size_t imageHeight, double pixelScaleX, double pixelScaleY, double minUVInLambda, double maxUVInLambda, double rankFilterLevel, size_t rankFilterSize, bool weightsAsTaper) :
 		_weightMode(weightMode),
 		_imageWidth(imageWidth),
 		_imageHeight(imageHeight),
@@ -26,6 +26,7 @@ public:
 		_tukeyTaperInLambda(0), _tukeyInnerTaperInLambda(0),
 		_edgeTaperInLambda(0),
 		_edgeTukeyTaperInLambda(0),
+		_weightsAsTaper(weightsAsTaper),
 		_currentWeightChannel(std::numeric_limits<size_t>::max()),
 		_currentWeightInterval(std::numeric_limits<size_t>::max())
 	{
@@ -40,20 +41,20 @@ public:
 		_edgeTukeyTaperInLambda = edgeTukeyTaperInLambda;
 	}
 	
-	void Update(MeasurementSetGridder& gridder, size_t outChannelIndex, size_t outIntervalIndex)
+	void Update(const std::vector<std::pair<std::unique_ptr<MSProvider>, MSSelection>>& msList, size_t outChannelIndex, size_t outIntervalIndex)
 	{
 		if(outChannelIndex != _currentWeightChannel || outIntervalIndex != _currentWeightInterval)
 		{
 			_currentWeightChannel = outChannelIndex;
 			_currentWeightInterval = outIntervalIndex;
 			
-			recalculateWeights(gridder);
+			recalculateWeights(msList);
 		}
 	}
 	
 	void ResetWeights()
 	{
-		_imageWeights.reset(new ImageWeights(_weightMode, _imageWidth, _imageHeight, _pixelScaleX, _pixelScaleY, _weightMode.SuperWeight()));
+		_imageWeights.reset(new ImageWeights(_weightMode, _imageWidth, _imageHeight, _pixelScaleX, _pixelScaleY, _weightsAsTaper, _weightMode.SuperWeight()));
 	};
 	
 	ImageWeights& Weights()
@@ -91,15 +92,15 @@ public:
 	}
 
 private:
-	void recalculateWeights(MeasurementSetGridder& gridder)
+	void recalculateWeights(const std::vector<std::pair<std::unique_ptr<MSProvider>, MSSelection>>& msList)
 	{
 		Logger::Info << "Precalculating weights for " << _weightMode.ToString() << " weighting... ";
 		Logger::Info.Flush();
 		ResetWeights();
-		for(size_t i=0; i!=gridder.MeasurementSetCount(); ++i)
+		for(size_t i=0; i!=msList.size(); ++i)
 		{
-			_imageWeights->Grid(gridder.MeasurementSet(i), gridder.Selection(i));
-			if(gridder.MeasurementSetCount() > 1)
+			_imageWeights->Grid(*msList[i].first, msList[i].second);
+			if(msList.size() > 1)
 				(Logger::Info << i << ' ').Flush();
 		}
 		_imageWeights->FinishGridding();
@@ -118,6 +119,7 @@ private:
 	double _tukeyTaperInLambda, _tukeyInnerTaperInLambda;
 	double _edgeTaperInLambda;
 	double _edgeTukeyTaperInLambda;
+	bool _weightsAsTaper;
 	
 	size_t _currentWeightChannel, _currentWeightInterval;
 };
