@@ -2,6 +2,7 @@
 #define MEASUREMENT_SET_GRIDDER_H
 
 #include "gridmodeenum.h"
+#include "imagebufferallocator.h"
 
 #include "../polarization.h"
 #include "../msselection.h"
@@ -31,9 +32,11 @@ class MeasurementSetGridder
 			_imageHeight(0),
 			_trimWidth(0), _trimHeight(0),
 			_nwWidth(0), _nwHeight(0),
+			_nwFactor(1.0),
 			_pixelSizeX((1.0 / 60.0) * M_PI / 180.0),
 			_pixelSizeY((1.0 / 60.0) * M_PI / 180.0),
 			_wGridSize(0),
+			_actualWGridSize(0),
 			_measurementSets(),
 			_dataColumnName("DATA"),
 			_doImagePSF(false),
@@ -41,14 +44,13 @@ class MeasurementSetGridder
 			_addToModel(false),
 			_smallInversion(false),
 			_wLimit(0.0),
-			_precalculatedWeightInfo(0),
+			_precalculatedWeightInfo(nullptr),
 			_polarization(Polarization::StokesI),
 			_isComplex(false),
 			_weighting(WeightMode::UniformWeighted),
 			_verbose(false),
 			_antialiasingKernelSize(7),
 			_overSamplingFactor(63),
-			_normalizeForWeighting(true),
 			_visibilityWeightingMode(NormalVisibilityWeighting),
 			_gridMode(KaiserBesselKernel),
 			_storeImagingWeights(false)
@@ -64,6 +66,7 @@ class MeasurementSetGridder
 		double PixelSizeY() const { return _pixelSizeY; }
 		bool HasWGridSize() const { return _wGridSize != 0; }
 		size_t WGridSize() const { return _wGridSize; }
+		size_t ActualWGridSize() const { return _actualWGridSize; }
 		
 		void ClearMeasurementSetList() { _measurementSets.clear(); _selections.clear(); }
 		class MSProvider& MeasurementSet(size_t index) const { return *_measurementSets[index]; }
@@ -112,6 +115,10 @@ class MeasurementSetGridder
 		{
 			_wGridSize = wGridSize;
 		}
+		void SetActualWGridSize(size_t actualWGridSize)
+		{
+			_actualWGridSize = actualWGridSize;
+		}
 		void SetNoWGridSize()
 		{
 			_wGridSize = 0;
@@ -179,11 +186,11 @@ class MeasurementSetGridder
 		
 		virtual void Invert() = 0;
 		
-		virtual void Predict(double* image) = 0;
-		virtual void Predict(double* real, double* imaginary) = 0;
+		virtual void Predict(ImageBufferAllocator::Ptr image) = 0;
+		virtual void Predict(ImageBufferAllocator::Ptr real, ImageBufferAllocator::Ptr imaginary) = 0;
 		
-		virtual double *ImageRealResult() = 0;
-		virtual double *ImageImaginaryResult() = 0;
+		virtual ImageBufferAllocator::Ptr ImageRealResult() = 0;
+		virtual ImageBufferAllocator::Ptr ImageImaginaryResult() = 0;
 		virtual double PhaseCentreRA() const = 0;
 		virtual double PhaseCentreDec() const = 0;
 		virtual bool HasDenormalPhaseCentre() const { return false; }
@@ -224,16 +231,19 @@ class MeasurementSetGridder
 		}
 		size_t NWWidth() const { return _nwWidth; }
 		size_t NWHeight() const { return _nwHeight; }
+		double NWFactor() const { return _nwFactor; }
 		void SetNWSize(size_t nwWidth, size_t nwHeight) {
 			_nwWidth = nwWidth;
 			_nwHeight = nwHeight;
 		}
+		void SetNWFactor(double factor) { _nwFactor = factor; }
 	private:
 		size_t _imageWidth, _imageHeight;
 		size_t _trimWidth, _trimHeight;
 		size_t _nwWidth, _nwHeight;
+		double _nwFactor;
 		double _pixelSizeX, _pixelSizeY;
-		size_t _wGridSize;
+		size_t _wGridSize, _actualWGridSize;
 		std::vector<MSProvider*> _measurementSets;
 		std::string _dataColumnName;
 		bool _doImagePSF, _doSubtractModel, _addToModel, _smallInversion;
@@ -245,7 +255,6 @@ class MeasurementSetGridder
 		bool _verbose;
 		std::vector<MSSelection> _selections;
 		size_t _antialiasingKernelSize, _overSamplingFactor;
-		bool _normalizeForWeighting;
 		enum VisibilityWeightingMode _visibilityWeightingMode;
 		GridModeEnum _gridMode;
 		bool _storeImagingWeights;
